#
# Linear search of sorted list
#

import random
import time

def make_list(N, sorted=False):
  a = [1] * N
  for i in range(1, N):
    a[i] = a[i-1] + random.randrange(1, 100)
  if not sorted:
    random.shuffle(a)
  return a

def python_search(a, x):
  return a.index(x)

def linear_search(a, x):
  for i in range(len(a)):
    if a[i] == x:
      return i
  raise ValueError(x)

def sorted_linear_search1(a, x):
  for i in range(len(a)):
    if a[i] == x:
      return i
    if a[i] > x:
      raise ValueError(x)
  raise ValueError(x)

# If x is not in the list a, return position where it should be inserted
def sorted_linear_search(a, x):
  for i in range(len(a)):
    if a[i] >= x:
      return i
  return len(a)

def timing(alg, a, x):
  startTime = time.time()
  totalTime = 0
  rounds = 0
  while totalTime < 1.0:
    try:
      result = alg(a, x)
    except:
      result = -1
    totalTime = time.time() - startTime
    rounds += 1
  print("N = %6d time = %9d microsecs" % 
        (len(a), totalTime * 1000000 // rounds))
  return result


def main():
  N = 1000000
  a = make_list(N, sorted=True)
  print("The list:")
  print(a[:5])
  print(a[-5:])
  el = a[200] + 1
  print("Element", el, "appears at index", timing(python_search, a, el))
  print("Element", el, "appears at index", timing(linear_search, a, el))
  print("Element", el, "appears at index", timing(sorted_linear_search, a, el))

main()
